// SPDX-FileCopyrightText: OpenTalk GmbH <mail@opentalk.eu>
//
// SPDX-License-Identifier: EUPL-1.2
import { styled, Box as MuiBox } from '@mui/material';
import { MediaSessionType, ParticipantId } from '@opentalk/common';
import React from 'react';
import { useMemo, useState } from 'react';

import NameTile from '../../commonComponents/NameTile/NameTile';
import { useAppSelector } from '../../hooks';
import { useFullscreenContext } from '../../provider/FullscreenProvider';
import { selectSubscriberById } from '../../store/slices/mediaSubscriberSlice';
import { selectParticipantName } from '../../store/slices/participantsSlice';
import HandRaisedIndicator from './fragments/HandRaisedIndicator';
import ParticipantVideo from './fragments/ParticipantVideo';
import VideoOverlay from './fragments/VideoOverlay';

const Container = styled(MuiBox)(({ theme }) => ({
  position: 'relative',
  width: 'inherit',
  height: 'inherit',
  maxWidth: '100%',
  maxHeight: '100%',
  display: 'flex',
  flexWrap: 'nowrap',
  alignItems: 'center',
  justifyContent: 'center',
  background: theme.palette.background.video,
}));

const HandRaisedBox = styled(MuiBox)({
  position: 'absolute',
  right: 5,
  bottom: 5,
  display: 'flex',
  alignItems: 'flex-end',
  justifyContent: 'flex-end',
});

interface ParticipantWindowProps {
  participantId: ParticipantId;
  activePresenter?: boolean;
  alwaysShowOverlay?: boolean;
  isThumbnail?: boolean;
}

const ParticipantWindow = ({
  participantId,
  activePresenter,
  alwaysShowOverlay,
  isThumbnail,
}: ParticipantWindowProps) => {
  const fullscreenHandle = useFullscreenContext();
  const displayName = useAppSelector(selectParticipantName(participantId));
  const [activeOverlay, setActiveOverlay] = useState<boolean>(!!alwaysShowOverlay);

  const videoDescriptor = useMemo(
    () => ({ participantId: participantId, mediaType: MediaSessionType.Video }),
    [participantId]
  );
  const videoSubscriber = useAppSelector(selectSubscriberById(videoDescriptor));

  const handleDisplayOverlay = (show: boolean) => !alwaysShowOverlay && setActiveOverlay(show);

  return (
    <Container
      onMouseEnter={() => handleDisplayOverlay(true)}
      onMouseLeave={() => handleDisplayOverlay(false)}
      data-testid="ParticipantWindow"
    >
      <ParticipantVideo
        participantId={participantId}
        presenterVideoIsActive={activePresenter}
        isThumbnail={isThumbnail}
      />
      <VideoOverlay participantId={participantId} active={activeOverlay && !fullscreenHandle.active} />
      {!fullscreenHandle.active && (
        <NameTile audioOn={!!videoSubscriber?.audio} displayName={displayName || ''} className="positionBottom" />
      )}
      <HandRaisedBox>
        <HandRaisedIndicator participantId={participantId} />
      </HandRaisedBox>
    </Container>
  );
};

export default ParticipantWindow;
